@@ -77,7 +77,7 @@ debconf_select () {
 		else
 			debconf_select_lead="> "
 		fi
-		option=$(echo "${x#*$TAB}" | sed 's/ *$//g' | sed "s/^ /$debconf_select_lead/g")
+		option=$(echo "${x#*$TAB}" | sed "s/ *\$//g; s/^ /$debconf_select_lead/g")
 		newchoices="${newchoices:+${newchoices}${NL}}${key}${TAB}${option}"
 		if [ "$key" = "$default_choice" ]; then
 		    default="$option"
@@ -94,10 +94,16 @@ debconf_select () {
 	# escape the commas and leading whitespace but keep them unescaped
 	# in $choices
         for x in $choices; do
-                u="$u, `echo ${x#*$TAB} | sed 's/,/\\\\,/g' | sed 's/^ /\\\\ /'`"
+                u="$u, `echo ${x#*$TAB} | sed 's/,/\\\\,/g; s/^ /\\\\ /'`"
         done
         u=${u#, }
 	restore_ifs
+	# TODO: This can be preseeded without having to use translated
+	# values (which are often inappropriate for preseeding across many
+	# machines due to including e.g. disk capacities) but it's nasty;
+	# you have to use runes like
+	# "20some_device__________/var/lib/partman/devices/=dev=sda". We
+	# could do with an abbreviated syntax.
 	if [ -n "$default" ]; then
 	        db_set $template "$default"
 	fi
@@ -526,6 +532,22 @@ humandev () {
 		printf "$RET" ${idenum} ${linux}
 	    fi
 	    ;;
+	# Some drivers advertise the disk as "part", workaround for #404950
+	/dev/ide/host*/bus[01]/target[01]/lun0/part)
+	    host=`echo $1 | sed 's,/dev/ide/host\(.*\)/bus.*/target[01]/lun0/part,\1,'`
+	    bus=`echo $1 | sed 's,/dev/ide/host.*/bus\(.*\)/target[01]/lun0/part,\1,'`
+	    target=`echo $1 | sed 's,/dev/ide/host.*/bus.*/target\([01]\)/lun0/part,\1,'`
+	    idenum=$((2 * $host + $bus + 1))
+	    linux=$(mapdevfs $1)
+	    linux=${linux#/dev/}
+	    if [ "$target" = 0 ]; then
+		db_metaget partman/text/ide_master_disk description
+		printf "$RET" ${idenum} ${linux}
+	    else
+		db_metaget partman/text/ide_slave_disk description
+		printf "$RET" ${idenum} ${linux}
+	    fi
+	    ;;
 	/dev/ide/host*/bus[01]/target[01]/lun0/part*)
 	    host=`echo $1 | sed 's,/dev/ide/host\(.*\)/bus.*/target[01]/lun0/part.*,\1,'`
 	    bus=`echo $1 | sed 's,/dev/ide/host.*/bus\(.*\)/target[01]/lun0/part.*,\1,'`
@@ -690,7 +712,7 @@ humandev () {
 		printf "$RET" ".CCISS" "-" "$controller" "$lun" "$part" "$linux"
 	    fi
 	    ;;
-	/dev/md/*)
+	/dev/md*|/dev/md/*)
 	    device=`echo "$1" | sed -e "s/.*md\/\?\(.*\)/\1/"`
 	    type=`grep "^md${device}[ :]" /proc/mdstat | sed -e "s/^.* : active raid\([[:alnum:]]\).*/\1/"`
 	    db_metaget partman/text/raid_device description
@@ -703,7 +725,7 @@ humandev () {
 	        type=$(/sbin/dmsetup table "$1" | head -n 1 | cut -d " " -f3)
 	    fi
 
-	    if [ $type = crypt ]; then
+	    if [ "$type" = crypt ]; then
 	        mapping=${1#/dev/mapper/}
 	        db_metaget partman/text/dmcrypt_volume description
 	        printf "$RET" $mapping
@@ -714,7 +736,7 @@ humandev () {
 	        # occurance of one single dash to split the string into vg and
 	        # lv, and then replace two dashes next to each other with one.
 	        vglv=${1#/dev/mapper/}
-	        vglv=`echo "$vglv" | sed -e 's/\([^-]\)-\([^-]\)/\1 \2/' | sed -e 's/--/-/g'`
+	        vglv=`echo "$vglv" | sed 's/\([^-]\)-\([^-]\)/\1 \2/; s/--/-/g'`
 	        vg=`echo "$vglv" | cut -d" " -f1`
 	        lv=`echo "$vglv" | cut -d" " -f2`
 	        db_metaget partman/text/lvm_lv description
@@ -793,7 +815,9 @@ enable_swap () {
 	close_dialog
     done
     for path in $swaps; do
-	swapon $path || true
+	if ! grep -q "$path" /proc/swaps; then
+	    swapon $path 2>/dev/null || true
+	fi
     done
     cd "$startdir"
 }
@@ -824,34 +848,50 @@ default_disk_label () {
 	    else
 		echo msdos
 	    fi;;	    
-	arm)
+	arm|armel)
 	    case "$sub" in
+		iop32x)
+		    echo msdos;;
+		iop33x)
+		    echo msdos;;
+		ixp4xx)
+		    echo msdos;;
 		riscstation)
 		    echo msdos;;
 		netwinder)
 		    echo msdos;;
 		ads)
 		    echo msdos;;
-		nslu2)
+		versatile)
 		    echo msdos;;
 		*)
 		    echo UNKNOWN;;
 	    esac;;
 	armeb)
 	    case "$sub" in
-		nslu2)
+		ixp4xx)
 		    echo msdos;;
 		*)
 		    echo UNKNOWN;;
 	    esac;;
 	amd64)
-	    echo msdos;;
+	    case "$sub" in
+		mac)
+		    echo gpt;;
+		*)
+		    echo msdos;;
+	    esac;;
 	hppa)
 	    echo msdos;;
 	ia64)
 	    echo gpt;;
 	i386)
-	    echo msdos;;
+	    case "$sub" in
+		mac)
+		    echo gpt;;
+		*)
+		    echo msdos;;
+	    esac;;
 	m68k)
 	    case "$sub" in
 		amiga)
@@ -883,6 +923,8 @@ default_disk_label () {
 		# Broadcom SB1 evaluation boards
 		sb1-bcm91250a | sb1a-bcm91480b)
 		    echo msdos;;
+		qemu-mips32)
+		    echo msdos;;
 		*)
 		    echo UNKNOWN;;
 	    esac;;
@@ -900,6 +942,8 @@ default_disk_label () {
 		    echo msdos;;
 		bcm947xx)
 		    echo msdos;;
+		qemu-mips32)
+		    echo msdos;;
 		*)
 		    echo UNKNOWN;;
 	    esac;;
@@ -921,6 +965,8 @@ default_disk_label () {
 		    echo mac;;
 		powermac_oldworld)
 		    echo mac;;
+		ps3)
+		    echo msdos;;
 		*)
 		    echo UNKNOWN;;
 	    esac;;
@@ -996,7 +1042,7 @@ partman_unlock_unit() {
 
 # List the changes that are about to be committed and let the user confirm first
 confirm_changes () {
-	local template dev x part partitions num id size type fs path name filesystem partitems items formatted_previously
+	local template dev x part partitions num id size type fs path name filesystem partdesc partitems items formatted_previously
 	template="$1"
 
 	# Compute the changes we are going to do
@@ -1042,10 +1088,16 @@ confirm_changes () {
 				continue
 			}
 			filesystem=$(cat $id/visual_filesystem)
-			db_subst partman/text/confirm_item TYPE "$filesystem"
-			db_subst partman/text/confirm_item PARTITION "$num"
-			db_subst partman/text/confirm_item DEVICE $(humandev $(cat device))
-			db_metaget partman/text/confirm_item description
+			# Special case d-m devices to use a different description
+			if cat device | grep -q "/dev/mapper" ; then
+				partdesc="partman/text/confirm_unpartitioned_item"
+			else
+				partdesc="partman/text/confirm_item"
+				db_subst $partdesc PARTITION "$num"
+			fi
+			db_subst $partdesc TYPE "$filesystem"
+			db_subst $partdesc DEVICE $(humandev $(cat device))
+			db_metaget $partdesc description
 		    
 			items="${items}   ${RET}
 "
