@@ -33,9 +33,9 @@ import socket
 import fcntl
 import traceback
 import syslog
+import gzip
 import debconf
 import apt_pkg
-from apt.package import Package
 from apt.cache import Cache
 from apt.progress import FetchProgress, InstallProgress
 
@@ -44,7 +44,8 @@ sys.path.insert(0, '/usr/lib/ubiquity')
 from ubiquity import misc
 from ubiquity.components import language_apply, apt_setup, timezone_apply, \
                                 clock_setup, console_setup_apply, \
-                                usersetup_apply, hw_detect, check_kernels
+                                usersetup_apply, hw_detect, check_kernels, \
+                                migrationassistant_apply
 
 class DebconfFetchProgress(FetchProgress):
     """An object that reports apt's fetching progress using debconf."""
@@ -304,9 +305,9 @@ class Install:
             self.db.progress('REGION', 1, 75)
             try:
                 self.copy_all()
-            except OSError, e:
-                if e.errno in (errno.ENOENT, errno.EIO, errno.ENOTDIR,
-                               errno.EROFS):
+            except EnvironmentError, e:
+                if e.errno in (errno.ENOENT, errno.EIO, errno.EFAULT,
+                               errno.ENOTDIR, errno.EROFS):
                     if e.filename is None:
                         error_template = 'cd_hd_fault'
                     elif e.filename.startswith('/target'):
@@ -366,10 +367,15 @@ class Install:
             self.configure_timezone()
 
             self.db.progress('SET', 86)
-            self.db.progress('REGION', 86, 88)
+            self.db.progress('REGION', 86, 87)
             self.db.progress('INFO', 'ubiquity/install/keyboard')
             self.configure_keyboard()
 
+            self.db.progress('SET', 87)
+            self.db.progress('REGION', 87, 88)
+            self.db.progress('INFO', 'ubiquity/install/migrationassistant')
+            self.configure_ma()
+
             self.db.progress('SET', 88)
             self.db.progress('REGION', 88, 89)
             self.remove_unusable_kernels()
@@ -546,6 +552,19 @@ class Install:
                 syslog.syslog(syslog.LOG_ERR,
                               'Failed to copy installation log file')
             os.chmod(target_log_file, stat.S_IRUSR | stat.S_IWUSR)
+        try:
+            status = open(os.path.join(self.target, 'var/lib/dpkg/status'))
+            status_gz = gzip.open(os.path.join(target_dir,
+                                               'initial-status.gz'), 'w')
+            while True:
+                data = status.read(65536)
+                if not data:
+                    break
+                status_gz.write(data)
+            status_gz.close()
+            status.close()
+        except IOError:
+            pass
 
 
     def mount_one_image(self, fsfile, mountpoint=None):
@@ -607,12 +626,15 @@ class Install:
 
         if fs_preseed == '':
             # Simple autodetection on unionfs systems
-            for line in open('/proc/mounts'):
+            mounts = open('/proc/mounts')
+            for line in mounts:
                 (device, fstype) = line.split()[1:3]
                 if fstype == 'squashfs' and os.path.exists(device):
                     misc.ex('mount', '--bind', device, self.source)
                     self.mountpoints.append(self.source)
+                    mounts.close()
                     return
+            mounts.close()
 
             # Manual detection on non-unionfs systems
             fsfiles = ['/cdrom/casper/filesystem.cloop',
@@ -671,6 +693,45 @@ class Install:
                     "Failed to detach loopback device %s" % dev)
 
 
+    def chroot_setup(self):
+        """Set up /target for safe package management operations."""
+        policy_rc_d = os.path.join(self.target, 'usr/sbin/policy-rc.d')
+        f = open(policy_rc_d, 'w')
+        print >>f, """\
+#!/bin/sh
+exit 101"""
+        f.close()
+        os.chmod(policy_rc_d, 0755)
+
+        start_stop_daemon = os.path.join(self.target, 'sbin/start-stop-daemon')
+        if os.path.exists(start_stop_daemon):
+            os.rename(start_stop_daemon, '%s.REAL' % start_stop_daemon)
+        f = open(start_stop_daemon, 'w')
+        print >>f, """\
+#!/bin/sh
+echo 1>&2
+echo 'Warning: Fake start-stop-daemon called, doing nothing.' 1>&2
+exit 0"""
+        f.close()
+        os.chmod(start_stop_daemon, 0755)
+
+        if not os.path.exists(os.path.join(self.target, 'proc/cmdline')):
+            self.chrex('mount', '-t', 'proc', 'proc', '/proc')
+        if not os.path.exists(os.path.join(self.target, 'sys/devices')):
+            self.chrex('mount', '-t', 'sysfs', 'sysfs', '/sys')
+
+    def chroot_cleanup(self):
+        """Undo the work done by chroot_setup."""
+        self.chrex('umount', '/sys')
+        self.chrex('umount', '/proc')
+
+        start_stop_daemon = os.path.join(self.target, 'sbin/start-stop-daemon')
+        os.rename('%s.REAL' % start_stop_daemon, start_stop_daemon)
+
+        policy_rc_d = os.path.join(self.target, 'usr/sbin/policy-rc.d')
+        os.unlink(policy_rc_d)
+
+
     def run_target_config_hooks(self):
         """Run hook scripts from /usr/lib/ubiquity/target-config. This allows
         casper to hook into us and repeat bits of its configuration in the
@@ -682,14 +743,12 @@ class Install:
             # Exclude hooks containing '.', so that *.dpkg-* et al are avoided.
             hooks = filter(lambda entry: '.' not in entry, os.listdir(hookdir))
             self.db.progress('START', 0, len(hooks), 'ubiquity/install/title')
+            self.db.progress('INFO', 'ubiquity/install/target_hooks')
             for hookentry in hooks:
                 hook = os.path.join(hookdir, hookentry)
                 if not os.access(hook, os.X_OK):
                     self.db.progress('STEP', 1)
                     continue
-                self.db.subst('ubiquity/install/target_hook',
-                              'SCRIPT', hookentry)
-                self.db.progress('INFO', 'ubiquity/install/target_hook')
                 # Errors are ignored at present, although this may change.
                 subprocess.call(['log-output', '-t', 'ubiquity',
                                  '--pass-stdout', hook])
@@ -759,7 +818,14 @@ class Install:
             except SystemError:
                 apt_error = True
             if cache._depcache.BrokenCount > 0 or apt_error:
-                cachedpkg.markKeep()
+                brokenpkgs = self.broken_packages(cache)
+                while brokenpkgs:
+                    for brokenpkg in brokenpkgs:
+                        self.get_cache_pkg(cache, brokenpkg).markKeep()
+                    new_brokenpkgs = self.broken_packages(cache)
+                    if brokenpkgs == new_brokenpkgs:
+                        break # we can do nothing more
+                    brokenpkgs = new_brokenpkgs
                 assert cache._depcache.BrokenCount == 0
 
 
@@ -811,6 +877,14 @@ class Install:
             'ubiquity/install/apt_indices_starting',
             'ubiquity/install/apt_indices')
         cache = Cache()
+
+        if cache._depcache.BrokenCount > 0:
+            syslog.syslog(
+                'not installing language packs, since there are broken '
+                'packages: %s' % ', '.join(self.broken_packages(cache)))
+            self.db.progress('STOP')
+            return
+
         try:
             # update() returns False on failure and 0 on success. Madness!
             if cache.update(fetchprogress) not in (0, True):
@@ -841,6 +915,7 @@ class Install:
                 installed_pkgs.append(pkg)
         self.record_installed(installed_pkgs)
 
+        commit_error = None
         try:
             if not cache.commit(fetchprogress, installprogress):
                 fetchprogress.stop()
@@ -850,15 +925,27 @@ class Install:
         except IOError, e:
             for line in str(e).split('\n'):
                 syslog.syslog(syslog.LOG_WARNING, line)
-            self.db.progress('STOP')
-            raise
+            commit_error = str(e)
         except SystemError, e:
             for line in str(e).split('\n'):
                 syslog.syslog(syslog.LOG_WARNING, line)
-            self.db.progress('STOP')
-            raise
+            commit_error = str(e)
         self.db.progress('SET', 100)
 
+        cache.open(None)
+        if commit_error or cache._depcache.BrokenCount > 0:
+            if commit_error is None:
+                commit_error = ''
+            brokenpkgs = self.broken_packages(cache)
+            syslog.syslog('broken packages after language pack installation: '
+                          '%s' % ', '.join(brokenpkgs))
+            self.db.subst('ubiquity/install/broken_install', 'ERROR',
+                          commit_error)
+            self.db.subst('ubiquity/install/broken_install', 'PACKAGES',
+                          ', '.join(brokenpkgs))
+            self.db.input('critical', 'ubiquity/install/broken_install')
+            self.db.go()
+
         self.db.progress('STOP')
 
 
@@ -895,6 +982,16 @@ class Install:
         ret = dbfilter.run_command(auto_process=True)
         if ret != 0:
             raise InstallStepError("UserSetupApply failed with code %d" % ret)
+    
+    def configure_ma(self):
+        """import documents, settings, and users from previous operating
+        systems."""
+
+        if 'UBIQUITY_MIGRATION_ASSISTANT' in os.environ:
+            dbfilter = migrationassistant_apply.MigrationAssistantApply(None)
+            ret = dbfilter.run_command(auto_process=True)
+            if ret != 0:
+                raise InstallStepError("MigrationAssistantApply failed with code %d" % ret)
 
 
     def get_resume_partition(self):
@@ -917,10 +1014,14 @@ class Install:
         hardware system in which has been installed on and need some
         automatic configurations to get work."""
 
-        dbfilter = hw_detect.HwDetect(None, self.db)
-        ret = dbfilter.run_command(auto_process=True)
-        if ret != 0:
-            raise InstallStepError("HwDetect failed with code %d" % ret)
+        self.chroot_setup()
+        try:
+            dbfilter = hw_detect.HwDetect(None, self.db)
+            ret = dbfilter.run_command(auto_process=True)
+            if ret != 0:
+                raise InstallStepError("HwDetect failed with code %d" % ret)
+        finally:
+            self.chroot_cleanup()
 
         self.db.progress('INFO', 'ubiquity/install/hardware')
 
@@ -962,10 +1063,13 @@ class Install:
             os.unlink('/target/etc/popularity-contest.conf')
         except OSError:
             pass
+        try:
+            participate = self.db.get('popularity-contest/participate')
+            self.set_debconf('popularity-contest/participate', participate)
+        except debconf.DebconfError:
+            pass
 
-        self.chrex('mount', '-t', 'proc', 'proc', '/proc')
-        self.chrex('mount', '-t', 'sysfs', 'sysfs', '/sys')
-
+        self.chroot_setup()
         self.chrex('dpkg-divert', '--package', 'ubiquity', '--rename',
                    '--quiet', '--add', '/usr/sbin/update-initramfs')
         try:
@@ -982,8 +1086,6 @@ class Install:
             for package in packages:
                 self.reconfigure(package)
         finally:
-            self.chrex('umount', '/proc')
-            self.chrex('umount', '/sys')
             try:
                 os.unlink('/target/usr/sbin/update-initramfs')
             except OSError:
@@ -991,6 +1093,7 @@ class Install:
             self.chrex('dpkg-divert', '--package', 'ubiquity', '--rename',
                        '--quiet', '--remove', '/usr/sbin/update-initramfs')
             self.chrex('update-initramfs', '-c', '-k', os.uname()[2])
+            self.chroot_cleanup()
 
         # Fix up kernel symlinks now that the initrd exists. Depending on
         # the architecture, these may be in / or in /boot.
@@ -1078,15 +1181,24 @@ class Install:
             hostname = self.db.get('netcfg/get_hostname')
         except debconf.DebconfError:
             hostname = ''
+        try:
+            domain = self.db.get('netcfg/get_domain')
+        except debconf.DebconfError:
+            domain = ''
         if hostname == '':
             hostname = 'ubuntu'
+
         fp = open(os.path.join(self.target, 'etc/hostname'), 'w')
         print >>fp, hostname
         fp.close()
 
         hosts = open(os.path.join(self.target, 'etc/hosts'), 'w')
         print >>hosts, "127.0.0.1\tlocalhost"
-        print >>hosts, "127.0.1.1\t%s" % hostname
+        if domain:
+            print >>hosts, "127.0.1.1\t%s.%s\t%s" % (hostname, domain,
+                                                     hostname)
+        else:
+            print >>hosts, "127.0.1.1\t%s" % hostname
         print >>hosts, textwrap.dedent("""\
 
             # The following lines are desirable for IPv6 capable hosts
@@ -1155,23 +1267,35 @@ class Install:
         misc.ex('mount', '--bind', '/proc', self.target + '/proc')
         misc.ex('mount', '--bind', '/dev', self.target + '/dev')
 
+        archdetect = subprocess.Popen(['archdetect'], stdout=subprocess.PIPE)
+        subarch = archdetect.communicate()[0].strip()
+
         try:
-            from ubiquity.components import grubinstaller
-            dbfilter = grubinstaller.GrubInstaller(None)
-            ret = dbfilter.run_command(auto_process=True)
-            if ret != 0:
-                raise InstallStepError(
-                    "GrubInstaller failed with code %d" % ret)
-        except ImportError:
-            try:
+            if subarch.startswith('amd64/') or subarch.startswith('i386/'):
+                from ubiquity.components import grubinstaller
+                dbfilter = grubinstaller.GrubInstaller(None)
+                ret = dbfilter.run_command(auto_process=True)
+                if ret != 0:
+                    raise InstallStepError(
+                        "GrubInstaller failed with code %d" % ret)
+            elif subarch == 'powerpc/ps3':
+                from ubiquity.components import kbootinstaller
+                dbfilter = kbootinstaller.KbootInstaller(None)
+                ret = dbfilter.run_command(auto_process=True)
+                if ret != 0:
+                    raise InstallStepError(
+                        "KbootInstaller failed with code %d" % ret)
+            elif subarch.startswith('powerpc/'):
                 from ubiquity.components import yabootinstaller
                 dbfilter = yabootinstaller.YabootInstaller(None)
                 ret = dbfilter.run_command(auto_process=True)
                 if ret != 0:
                     raise InstallStepError(
                         "YabootInstaller failed with code %d" % ret)
-            except ImportError:
+            else:
                 raise InstallStepError("No bootloader installer found")
+        except ImportError:
+            raise InstallStepError("No bootloader installer found")
 
         misc.ex('umount', '-f', self.target + '/proc')
         misc.ex('umount', '-f', self.target + '/dev')
@@ -1180,8 +1304,12 @@ class Install:
     def broken_packages(self, cache):
         brokenpkgs = set()
         for pkg in cache.keys():
-            if cache._depcache.IsInstBroken(cache._cache[pkg]):
-                brokenpkgs.add(pkg)
+            try:
+                if cache._depcache.IsInstBroken(cache._cache[pkg]):
+                    brokenpkgs.add(pkg)
+            except KeyError:
+                # Apparently sometimes the cache goes a bit bonkers ...
+                continue
         return brokenpkgs
 
     def do_remove(self, to_remove, recursive=False):
@@ -1194,6 +1322,13 @@ class Install:
             'ubiquity/install/apt_indices')
         cache = Cache()
 
+        if cache._depcache.BrokenCount > 0:
+            syslog.syslog(
+                'not processing removals, since there are broken packages: '
+                '%s' % ', '.join(self.broken_packages(cache)))
+            self.db.progress('STOP')
+            return
+
         while True:
             removed = set()
             for pkg in to_remove:
@@ -1254,19 +1389,37 @@ class Install:
         installprogress = DebconfInstallProgress(
             self.db, 'ubiquity/install/title', 'ubiquity/install/apt_info',
             'ubiquity/install/apt_error_remove')
+        self.chroot_setup()
+        commit_error = None
         try:
-            if not cache.commit(fetchprogress, installprogress):
-                fetchprogress.stop()
-                installprogress.finishUpdate()
-                self.db.progress('STOP')
-                return
-        except SystemError, e:
-            for line in str(e).split('\n'):
-                syslog.syslog(syslog.LOG_ERR, line)
-            self.db.progress('STOP')
-            raise
+            try:
+                if not cache.commit(fetchprogress, installprogress):
+                    fetchprogress.stop()
+                    installprogress.finishUpdate()
+                    self.db.progress('STOP')
+                    return
+            except SystemError, e:
+                for line in str(e).split('\n'):
+                    syslog.syslog(syslog.LOG_ERR, line)
+                commit_error = str(e)
+        finally:
+            self.chroot_cleanup()
         self.db.progress('SET', 5)
 
+        cache.open(None)
+        if commit_error or cache._depcache.BrokenCount > 0:
+            if commit_error is None:
+                commit_error = ''
+            brokenpkgs = self.broken_packages(cache)
+            syslog.syslog('broken packages after removal: '
+                          '%s' % ', '.join(brokenpkgs))
+            self.db.subst('ubiquity/install/broken_remove', 'ERROR',
+                          commit_error)
+            self.db.subst('ubiquity/install/broken_remove', 'PACKAGES',
+                          ', '.join(brokenpkgs))
+            self.db.input('critical', 'ubiquity/install/broken_remove')
+            self.db.go()
+
         self.db.progress('STOP')
 
 
@@ -1284,8 +1437,10 @@ class Install:
 
         remove_kernels = set()
         if os.path.exists("/var/lib/ubiquity/remove-kernels"):
-            for line in open("/var/lib/ubiquity/remove-kernels"):
+            remove_kernels_file = open("/var/lib/ubiquity/remove-kernels")
+            for line in remove_kernels_file:
                 remove_kernels.add(line.strip())
+            remove_kernels_file.close()
 
         if len(remove_kernels) == 0:
             self.db.progress('STOP')
@@ -1313,13 +1468,17 @@ class Install:
         if (os.path.exists("/cdrom/casper/filesystem.manifest-desktop") and
             os.path.exists("/cdrom/casper/filesystem.manifest")):
             desktop_packages = set()
-            for line in open("/cdrom/casper/filesystem.manifest-desktop"):
+            manifest = open("/cdrom/casper/filesystem.manifest-desktop")
+            for line in manifest:
                 if line.strip() != '' and not line.startswith('#'):
                     desktop_packages.add(line.split()[0])
+            manifest.close()
             live_packages = set()
-            for line in open("/cdrom/casper/filesystem.manifest"):
+            manifest = open("/cdrom/casper/filesystem.manifest")
+            for line in manifest:
                 if line.strip() != '' and not line.startswith('#'):
                     live_packages.add(line.split()[0])
+            manifest.close()
             difference = live_packages - desktop_packages
         else:
             difference = set()
@@ -1327,8 +1486,10 @@ class Install:
         # Keep packages we explicitly installed.
         apt_installed = set()
         if os.path.exists("/var/lib/ubiquity/apt-installed"):
-            for line in open("/var/lib/ubiquity/apt-installed"):
+            apt_installed_file = open("/var/lib/ubiquity/apt-installed")
+            for line in apt_installed_file:
                 apt_installed.add(line.strip())
+            apt_installed_file.close()
         difference -= apt_installed
 
         if len(difference) == 0:
