@@ -21,4 +21,4 @@ from ubiquity.filteredcommand import FilteredCommand
 
 class ConsoleSetupApply(FilteredCommand):
     def prepare(self):
-        return (['/usr/lib/ubiquity/console-setup/post-base-installer'], [])
+        return (['/usr/share/ubiquity/console-setup-apply'], [])
