@@ -55,7 +55,18 @@ class UserSetup(FilteredCommand):
                      escape=True)
         self.preseed('passwd/user-uid', '')
 
-        super(UserSetup, self).ok_handler()
+        # evand 2007-01-13: This is probably unnecessary as migration-assistant
+        # is run after user-setup in d-i, but it might be best to do this anyway
+        # to avoid potential future headaches.
+        import os
+        if 'UBIQUITY_MIGRATION_ASSISTANT' in os.environ and \
+            self.db.get('migration-assistant/partitions'):
+            q = 'migration-assistant/new-user/%s/' % username
+            self.preseed(q + 'fullname', fullname)
+            self.preseed(q + 'password', password, escape=True)
+            self.preseed(q + 'password-again', password_confirm, escape=True)
+
+        FilteredCommand.ok_handler(self)
 
     def error(self, priority, question):
         if question.startswith('passwd/username-'):
@@ -65,4 +76,4 @@ class UserSetup(FilteredCommand):
         else:
             self.frontend.error_dialog(self.description(question),
                                        self.extended_description(question))
-        return super(UserSetup, self).error(priority, question)
+        return FilteredCommand.error(self, priority, question)
