@@ -50,12 +50,19 @@ class FilteredCommand(object):
         self.current_question = None
         self.succeeded = False
 
+    @classmethod
+    def debug_enabled(self):
+        return ('UBIQUITY_DEBUG_CORE' in os.environ and
+                os.environ['UBIQUITY_DEBUG_CORE'] == '1')
+
     @classmethod
     def debug(self, fmt, *args):
-        if ('UBIQUITY_DEBUG_CORE' in os.environ and
-            os.environ['UBIQUITY_DEBUG_CORE'] == '1'):
+        if self.debug_enabled():
+            import time
+            # bizarre time formatting code per syslogd
+            time_str = time.ctime()[4:19]
             message = fmt % args
-            syslog.syslog(syslog.LOG_DEBUG, '%s: %s' % (PACKAGE, message))
+            print >>sys.stderr, '%s %s: %s' % (time_str, PACKAGE, message)
 
     def start(self, auto_process=False):
         self.status = None
@@ -105,9 +112,8 @@ class FilteredCommand(object):
     def wait(self):
         ret = self.dbfilter.wait()
 
-        if ret != 0:
-            # TODO: error message if ret != 10
-            self.debug("%s exited with code %d", self.command, ret)
+        # TODO: error message if ret != 0 and ret != 10
+        self.debug("%s exited with code %d", self.command, ret)
 
         self.cleanup()
 
@@ -153,6 +159,8 @@ class FilteredCommand(object):
         self.start(auto_process=auto_process)
         if auto_process:
             self.enter_ui_loop()
+            if self.status is None:
+                self.status = self.wait()
         else:
             while self.process_line():
                 pass
@@ -235,10 +243,13 @@ class FilteredCommand(object):
                        'utf-8', 'replace')
 
     def extended_description(self, question):
-        self.db.capb('escape')
+        old_capb = self.db.capb()
+        capb_list = old_capb.split()
+        capb_list.append('escape')
+        self.db.capb(' '.join(capb_list))
         data = unicode(self.db.metaget(question, 'extended_description'),
                        'utf-8', 'replace')
-        self.db.capb('')
+        self.db.capb(old_capb)
         return data
 
     def translate_to_c(self, question, value):
@@ -266,7 +277,10 @@ class FilteredCommand(object):
             value = self.escape(value)
         value = value.encode("UTF-8", "ignore")
         if escape:
-            self.db.capb('escape')
+            old_capb = self.db.capb()
+            capb_list = old_capb.split()
+            capb_list.append('escape')
+            self.db.capb(' '.join(capb_list))
         try:
             self.db.set(name, value)
         except debconf.DebconfError:
@@ -274,7 +288,7 @@ class FilteredCommand(object):
             self.db.set(name, value)
             self.db.subst(name, 'ID', name)
         if escape:
-            self.db.capb('')
+            self.db.capb(old_capb)
 
         if seen:
             self.db.fset(name, 'seen', 'true')
@@ -369,6 +383,5 @@ class FilteredCommand(object):
                                               progress_region_end)
 
 if __name__ == '__main__':
-    import sys
     fc = FilteredCommand()
     fc.run(sys.argv[1])
