@@ -1,9 +1,6 @@
 # -*- coding: utf-8 -*-
 
-import sys
 import os
-import gobject
-import glob
 import syslog
 
 import gettext
